@@ -1,18 +1,15 @@
+# frozen_string_literal: true
+
 require File.expand_path('../../../util/ini_file', __FILE__)
 
 Puppet::Type.type(:ini_setting).provide(:ruby) do
   def self.instances
-    # this code is here to support purging and the query-all functionality of the
-    # 'puppet resource' command, on a per-file basis.  Users
-    # can create a type for a specific config file with a provider that uses
-    # this as its parent and implements the method
-    # 'self.file_path', and that will provide the value for the path to the
-    # ini file (rather than needing to specify it on each ini setting
-    # declaration).  This allows 'purging' to be used to clear out
-    # all settings from a particular ini file except those included in
-    # the catalog.
+    desc '
+    Creates new ini_setting file, a specific config file with a provider that uses
+    this as its parent and implements the method
+    self.file_path, and that will provide the value for the path to the
+    ini file.'
     raise(Puppet::Error, 'Ini_settings only support collecting instances when a file path is hard coded') unless respond_to?(:file_path)
-
     # figure out what to do about the seperator
     ini_file  = Puppet::Util::IniFile.new(file_path, '=')
     resources = []
@@ -31,15 +28,36 @@ Puppet::Type.type(:ini_setting).provide(:ruby) do
   end
 
   def self.namevar(section_name, setting)
-    "#{section_name}/#{setting}"
+    setting.nil? ? section_name : "#{section_name}/#{setting}"
   end
 
   def exists?
-    !ini_file.get_value(section, setting).nil?
+    if setting.nil?
+      ini_file.section_names.include?(section)
+    elsif ini_file.section?(section)
+      !ini_file.get_value(section, setting).nil?
+    elsif resource.parameters.key?(:force_new_section_creation) && !resource[:force_new_section_creation]
+      # for backwards compatibility, if a user is using their own ini_setting
+      # types but does not have this parameter, we need to fall back to the
+      # previous functionality which was to create the section.  Anyone
+      # wishing to leverage this setting must define it in their provider
+      # type. See comments on
+      # https://github.com/puppetlabs/puppetlabs-inifile/pull/286
+      resource[:ensure] = :absent
+      resource[:force_new_section_creation]
+    elsif resource.parameters.key?(:force_new_section_creation) && resource[:force_new_section_creation]
+      !resource[:force_new_section_creation]
+    else
+      false
+    end
   end
 
   def create
-    ini_file.set_value(section, setting, separator, resource[:value])
+    if setting.nil? && resource[:value].nil?
+      ini_file.set_value(section)
+    else
+      ini_file.set_value(section, setting, separator, resource[:value])
+    end
     ini_file.save
     @ini_file = nil
   end
@@ -55,7 +73,11 @@ Puppet::Type.type(:ini_setting).provide(:ruby) do
   end
 
   def value=(_value)
-    ini_file.set_value(section, setting, separator, resource[:value])
+    if setting.nil? && resource[:value].nil?
+      ini_file.set_value(section)
+    else
+      ini_file.set_value(section, setting, separator, resource[:value])
+    end
     ini_file.save
   end
 
