@@ -1,7 +1,14 @@
+# frozen_string_literal: true
+
 require File.expand_path('../../../util/ini_file', __FILE__)
 require File.expand_path('../../../util/setting_value', __FILE__)
 
 Puppet::Type.type(:ini_subsetting).provide(:ruby) do
+  desc '
+  Creates new ini_subsetting file, a specific config file with a provider that uses
+  this as its parent and implements the method
+  self.file_path, and that will provide the value for the path to the
+  ini file.'
   def exists?
     setting_value.get_subsetting_value(subsetting, resource[:use_exact_match])
   end
@@ -19,7 +26,11 @@ Puppet::Type.type(:ini_subsetting).provide(:ruby) do
 
   def destroy
     setting_value.remove_subsetting(subsetting, resource[:use_exact_match])
-    ini_file.set_value(section, setting, key_val_separator, setting_value.get_value)
+    if setting_value.get_value.empty? && resource[:delete_if_empty]
+      ini_file.remove_setting(section, setting)
+    else
+      ini_file.set_value(section, setting, key_val_separator, setting_value.get_value)
+    end
     ini_file.save
     @ini_file = nil
     @setting_value = nil
