@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require File.expand_path('../external_iterator', __FILE__)
 require File.expand_path('../ini_file/section', __FILE__)
 
@@ -24,24 +26,14 @@ module Puppet::Util
       @key_val_separator = key_val_separator
       @section_names = []
       @sections_hash = {}
-      parse_file if File.file?(@path)
+      parse_file
     end
 
     def section_regex
       # Only put in prefix/suffix if they exist
-      # Also, if the prefix is '', the negated
-      # set match should be a match all instead.
       r_string = '^\s*'
       r_string += Regexp.escape(@section_prefix)
-      r_string += '('
-      if @section_prefix != ''
-        r_string += '[^'
-        r_string += Regexp.escape(@section_prefix)
-        r_string += ']'
-      else
-        r_string += '.'
-      end
-      r_string += '*)'
+      r_string += '(.*)'
       r_string += Regexp.escape(@section_suffix)
       r_string += '\s*$'
       %r{#{r_string}}
@@ -56,12 +48,18 @@ module Puppet::Util
       end
     end
 
+    def section?(section_name)
+      @sections_hash.key?(section_name)
+    end
+
     def get_value(section_name, setting)
       @sections_hash[section_name].get_value(setting) if @sections_hash.key?(section_name)
     end
 
-    def set_value(*args) # rubocop:disable Style/AccessorMethodName : Recomended alternative is a common value name
+    def set_value(*args)
       case args.size
+      when 1
+        section_name = args[0]
       when 3
         # Backwards compatible set_value function, See MODULES-5172
         (section_name, setting, value) = args
@@ -104,7 +102,7 @@ module Puppet::Util
         # was modified.
         section_index = @section_names.index(section_name)
         increment_section_line_numbers(section_index + 1)
-      else
+      elsif !setting.nil? || !value.nil?
         section.set_additional_setting(setting, value)
       end
     end
@@ -125,9 +123,17 @@ module Puppet::Util
       # was modified.
       section_index = @section_names.index(section_name)
       decrement_section_line_numbers(section_index + 1)
+
+      return unless section.empty?
+      # By convention, it's time to remove this newly emptied out section
+      lines.delete_at(section.start_line)
+      decrement_section_line_numbers(section_index + 1)
+      @section_names.delete_at(section_index)
+      @sections_hash.delete(section.name)
     end
 
     def save
+      global_empty = @sections_hash[''].empty? && @sections_hash[''].additional_settings.empty?
       File.open(@path, 'w') do |fh|
         @section_names.each_index do |index|
           name = @section_names[index]
@@ -138,16 +144,15 @@ module Puppet::Util
           whitespace_buffer = []
 
           if section.new_section? && !section.global?
-            fh.puts("\n#{@section_prefix}#{section.name}#{@section_suffix}")
+            if index == 1 && !global_empty || index > 1
+              fh.puts('')
+            end
+
+            fh.puts("#{@section_prefix}#{section.name}#{@section_suffix}")
           end
 
           unless section.new_section?
-            # don't add empty sections
-            if section.empty? && !section.global?
-              next
-            end
-
-            # write all of the pre-existing settings
+            # write all of the pre-existing lines
             (section.start_line..section.end_line).each do |line_num|
               line = lines[line_num]
 
@@ -155,7 +160,7 @@ module Puppet::Util
               # if they are at the end of a section, we can insert
               # any new settings *before* the final chunk of whitespace
               # lines.
-              if line =~ %r{^\s*$}
+              if line.match?(%r{^\s*$})
                 whitespace_buffer << line
               else
                 # If we get here, we've found a non-whitespace line.
@@ -216,17 +221,25 @@ module Puppet::Util
 
     def read_section(name, start_line, line_iter)
       settings = {}
-      end_line_num = nil
+      end_line_num = start_line
       min_indentation = nil
+      empty = true
       loop do
         line, line_num = line_iter.peek
-        return Section.new(name, start_line, end_line_num, settings, min_indentation) if line_num.nil? || @section_regex.match(line)
+        if line_num.nil? || @section_regex.match(line)
+          # the global section always exists, even when it's empty;
+          # when it's empty, we must be sure it's thought of as new,
+          # which is signalled with a nil ending line
+          end_line_num = nil if name == '' && empty
+          return Section.new(name, start_line, end_line_num, settings, min_indentation)
+        end
         if (match = @setting_regex.match(line))
           settings[match[2]] = match[4]
           indentation = match[1].length
           min_indentation = [indentation, min_indentation || indentation].min
         end
         end_line_num = line_num
+        empty = false
         line_iter.next
       end
     end
@@ -265,7 +278,7 @@ module Puppet::Util
       #  file; for now assuming that this type is only used on
       #  small-ish config files that can fit into memory without
       #  too much trouble.
-      File.readlines(path)
+      File.file?(path) ? File.readlines(path) : []
     end
 
     # This utility method scans through the lines for a section looking for
