@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 module Puppet::Util
   # This class can work with a list of subsettings inside
   # an ini file setting string to add, remove, extract and set their values.
@@ -47,7 +49,7 @@ module Puppet::Util
     # Get the resulting setting value by joining all the
     # subsettings, separator and quote characters.
     # @return [String]
-    def get_value # rubocop:disable Style/AccessorMethodName : Unable to alter without breaking code
+    def get_value
       value = @subsetting_items.join @subsetting_separator
       @quote_char + value + @quote_char
     end
